# Generated by Django 4.2.13 on 2024-06-20 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0031_remotesite_owner_modifiable"),
    ]

    operations = [
        migrations.AlterField(
            model_name="appsetting",
            name="value",
            field=models.CharField(
                blank=True, help_text="Value of the setting", null=True
            ),
        ),
    ]
