# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-05 13:05
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0010_update_appsetting'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='roleassignment',
            name='projectrole_project_b13795_idx',
        ),
        migrations.RemoveIndex(
            model_name='roleassignment',
            name='projectrole_user_id_0e46f7_idx',
        ),
    ]
