# Generated by Django 4.2.11 on 2024-03-21 12:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0028_populate_finder_role"),
        ("filesfolders", "0004_update_uuid"),
    ]

    operations = [
        migrations.AlterField(
            model_name="file",
            name="folder",
            field=models.ForeignKey(
                blank=True,
                help_text="Folder under which object exists (null if root folder)",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_children",
                to="filesfolders.folder",
            ),
        ),
        migrations.AlterField(
            model_name="file",
            name="project",
            field=models.ForeignKey(
                help_text="Project in which the object belongs",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_objects",
                to="projectroles.project",
            ),
        ),
        migrations.AlterField(
            model_name="folder",
            name="folder",
            field=models.ForeignKey(
                blank=True,
                help_text="Folder under which object exists (null if root folder)",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_children",
                to="filesfolders.folder",
            ),
        ),
        migrations.AlterField(
            model_name="folder",
            name="project",
            field=models.ForeignKey(
                help_text="Project in which the object belongs",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_objects",
                to="projectroles.project",
            ),
        ),
        migrations.AlterField(
            model_name="hyperlink",
            name="folder",
            field=models.ForeignKey(
                blank=True,
                help_text="Folder under which object exists (null if root folder)",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_children",
                to="filesfolders.folder",
            ),
        ),
        migrations.AlterField(
            model_name="hyperlink",
            name="project",
            field=models.ForeignKey(
                help_text="Project in which the object belongs",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_objects",
                to="projectroles.project",
            ),
        ),
    ]
