# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-04-11 15:58
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AdminAlert',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.CharField(help_text='Alert message to be shown for users', max_length=255)),
                ('description', models.TextField(blank=True, help_text='Full description of alert (optional, will be shown on a separate page)', null=True)),
                ('date_created', models.DateTimeField(auto_now_add=True, help_text='Alert creation timestamp')),
                ('date_expire', models.DateTimeField(help_text='Alert expiration timestamp')),
                ('active', models.BooleanField(default=True, help_text='Alert status (for disabling the alert before expiration)')),
                ('omics_uuid', models.UUIDField(default=uuid.uuid4, help_text='Adminalerts Omics UUID', unique=True)),
            ],
        ),
    ]
