# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-16 08:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sodarcache', '0002_make_user_optional'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='jsoncacheitem',
            index=models.Index(fields=['project', 'app_name', 'name'], name='sodarcache__project_e54b84_idx'),
        ),
    ]
