# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-30 06:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [("bgjobs", "0002_backgroundjoblogentry")]

    operations = [
        migrations.AddField(
            model_name="backgroundjob",
            name="date_created",
            field=models.DateTimeField(
                auto_now_add=True,
                default=django.utils.timezone.now,
                help_text="DateTime of creation",
            ),
            preserve_default=False,
        )
    ]
