# Generated by Django 4.2.11 on 2024-04-17 15:55

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0028_populate_finder_role"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("timeline", "0011_make_uuid_unique"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="ProjectEvent",
            new_name="TimelineEvent",
        ),
        migrations.RenameModel(
            old_name="ProjectEventObjectRef",
            new_name="TimelineEventObjectRef",
        ),
        migrations.RenameModel(
            old_name="ProjectEventStatus",
            new_name="TimelineEventStatus",
        ),
    ]
