# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-15 12:55
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0013_update_appsetting_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='appsetting',
            name='value_json',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict, help_text='Optional JSON value for the setting', null=True),
        ),
    ]
