
# This file was generated by 'versioneer.py' (0.29) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2024-08-08T16:34:20+0200",
 "dirty": false,
 "error": null,
 "full-revisionid": "42d2732f930ca135d76c63f6bfc8c467a0a4f0c2",
 "version": "1.0.1"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
