# Generated by Django 4.2.11 on 2024-04-18 08:18

import uuid

from django.db import migrations

def gen_uuid(apps, schema_editor):
    TimelineEventObjectRef = apps.get_model('timeline', 'TimelineEventObjectRef')
    for status in TimelineEventObjectRef.objects.all():
        status.sodar_uuid = uuid.uuid4()
        status.save(update_fields=['sodar_uuid'])

class Migration(migrations.Migration):

    dependencies = [
        ("timeline", "0013_timelineeventobjectref_sodar_uuid"),
    ]

    operations = [
        migrations.RunPython(gen_uuid, reverse_code=migrations.RunPython.noop),
    ]
