# Generated by Django 3.1.7 on 2021-03-16 15:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0018_update_jsonfield'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='public_guest_access',
            field=models.BooleanField(default=False, help_text='Allow public guest access for the project, also including unauthenticated users if allowed on the site'),
        ),
    ]
