# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-09-23 12:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0011_remove_indexes'),
    ]

    operations = [
        migrations.AddField(
            model_name='remotesite',
            name='user_display',
            field=models.BooleanField(default=True, help_text='RemoteSite visibility to users'),
        ),
        migrations.AlterField(
            model_name='remotesite',
            name='secret',
            field=models.CharField(help_text='Secret token for connecting to the source site', max_length=255, null=True),
        ),
    ]
