# Generated by Django 3.2.12 on 2022-04-07 15:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('timeline', '0007_allow_null_project'),
    ]

    operations = [
        migrations.AddField(
            model_name='projectevent',
            name='plugin',
            field=models.CharField(blank=True, help_text='Plugin to which the event is related (optional, if unset plugin with the name of the app is assumed)', max_length=255, null=True),
        ),
    ]
