# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-23 10:27
from __future__ import unicode_literals

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filesfolders', '0003_rename_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Filesfolders SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='folder',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Filesfolders SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='hyperlink',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Filesfolders SODAR UUID', unique=True),
        ),
    ]
