# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-20 15:29
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('timeline', '0003_rename_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='projectevent',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Event SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='projecteventobjectref',
            name='object_uuid',
            field=models.UUIDField(blank=True, help_text='Object SODAR UUID', null=True),
        ),
    ]
