# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-10-01 10:16
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0015_fix_appsetting_constraint'),
    ]

    operations = [
        migrations.AlterField(
            model_name='appsetting',
            name='app_plugin',
            field=models.ForeignKey(help_text='App to which the setting belongs', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='settings', to='djangoplugins.Plugin'),
        ),
    ]
