# Generated by Django 4.2.11 on 2024-04-18 08:24

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("timeline", "0014_populate_uuid"),
    ]

    operations = [
        migrations.AlterField(
            model_name="timelineeventobjectref",
            name="sodar_uuid",
            field=models.UUIDField(
                default=uuid.uuid4, help_text="Object reference SODAR UUID",
                unique=True
            ),
        ),
    ]
