# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-06-19 08:40
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0009_rename_projectsetting'),
    ]

    operations = [
        migrations.AddField(
            model_name='appsetting',
            name='user_modifiable',
            field=models.BooleanField(default=True, help_text='Setting visibility in forms'),
        ),
        migrations.AddField(
            model_name='appsetting',
            name='value_json',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict, help_text='Optional JSON value for the setting'),
        ),
    ]
