# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-17 13:20
from __future__ import unicode_literals

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adminalerts', '0004_rename_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='adminalert',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Adminalerts SODAR UUID', unique=True),
        ),
    ]
