# Generated by Django 3.1.7 on 2021-02-25 14:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sodarcache', '0003_add_index'),
    ]

    operations = [
        migrations.AlterField(
            model_name='jsoncacheitem',
            name='data',
            field=models.JSONField(default=dict, help_text='Cached data as JSON'),
        ),
    ]
