# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-01-03 11:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adminalerts', '0005_update_uuid'),
    ]

    operations = [
        migrations.AddField(
            model_name='adminalert',
            name='require_auth',
            field=models.BooleanField(default=True, help_text='Require authorization to view alert'),
        ),
    ]
